#!/bin/bash

SKIP_HW_DESC_TTY_IRQS=1
source /etc/mod-hardware-descriptor.env

# If we need to wait for usb to be plugged in before activating gadget mode, do nothing
# On such units, we can only register g_ether after the USB is plugged in
if [ "${USB_PLUGGED_IN_WAIT_REQUITED}" -eq 1 ] && [ -n "${STARTED_VIA_SYSTEMD}" ]; then
    exit 0
fi

if [ -n "${STARTED_VIA_SYSTEMD}" ] && [ -n "${USB_STARTUP_DELAY}" ] && [ "${USB_STARTUP_DELAY}" -gt 0 ]; then
    /bin/sleep ${USB_STARTUP_DELAY}
fi

if [ -f /data/enable-usb-multi-gadget ] && [ -e /proc/device-tree/compatible ]; then
    /sbin/modprobe libcomposite

    if [ -f /data/enable-usb-windows-compat ]; then
        /sbin/modprobe usb_f_rndis
        WINDOWS_COMPAT_MODE=1
    else
        WINDOWS_COMPAT_MODE=0
    fi

    # Read serial
    if [ -f /var/cache/mod/tag ]; then
        SERIAL=$(cat /var/cache/mod/tag)
    else
        SERIAL=$(mod-hardware tag)
    fi
    SERIAL=$(echo ${SERIAL} | awk 'sub("-","\n")' | tail -n 1 | sed 's/-//g')

    # Create MAC address suffix
    MAC=$(echo ${SERIAL} | tail -c 10 | sed 's/\(\w\w\)/:\1/g')

    # create new composite gadget instance
    mkdir /sys/kernel/config/usb_gadget/multi
    cd /sys/kernel/config/usb_gadget/multi

    # assign an unique id based on model, so each one is handled separately/differently by the host OS
    if [ "${PLATFORM}" = "duo" ]; then
        PRODUCT_CODE="1"
    elif [ "${PLATFORM}" = "dwarf" ]; then
        PRODUCT_CODE="2"
    elif [ -n "${MODEL_IMX8MQ}" ] && [ "${MODEL_IMX8MQ}" -eq 1 ]; then
        PRODUCT_CODE="3"
    else
        PRODUCT_CODE="4"
    fi

    # please increment this value everytime you make changes here!
    PRODUCT_VERSION="4"

    # make a unique device id based on previous values
    echo "0x3${PRODUCT_CODE}${PRODUCT_VERSION}${WINDOWS_COMPAT_MODE}" > bcdDevice

    # USB ID, see more at http://www.linux-usb.org/usb.ids
    echo "0x1d6b" > idVendor
    echo "0x0104" > idProduct

    # Setup USB Gadget
    echo 0x0200 > bcdUSB # USB 2.0
    echo 0xEF > bDeviceClass # Miscellaneous
    echo 0x02 > bDeviceSubClass
    echo 0x01 > bDeviceProtocol

    # USB Information (0x409 is English)
    mkdir strings/0x409
    echo "MOD Audio" > strings/0x409/manufacturer
    echo "${MODEL_NAME}" > strings/0x409/product
    echo "${SERIAL}" > strings/0x409/serialnumber

    # Documentation for available parameters at https://www.kernel.org/doc/html/v6.0/usb/gadget-testing.html

    # Setup MIDI
    mkdir functions/midi.usb0

    if [ ${WINDOWS_COMPAT_MODE} -eq 1 ]; then
        # Setup RNDIS
        mkdir functions/rndis.usb0
        echo 0xEF > functions/rndis.usb0/class
        echo 0x04 > functions/rndis.usb0/subclass
        echo 0x01 > functions/rndis.usb0/protocol
        echo "22${MAC}" > functions/rndis.usb0/dev_addr
        echo "32${MAC}" > functions/rndis.usb0/host_addr
        echo "RNDIS" > functions/rndis.usb0/os_desc/interface.rndis/compatible_id
        echo "5162001" > functions/rndis.usb0/os_desc/interface.rndis/sub_compatible_id

        ## Hacky stuff for Windows
        echo "1" > os_desc/use
        echo "0xcd" > os_desc/b_vendor_code
        echo "MSFT100" > os_desc/qw_sign
    else
        # Setup ECM
        mkdir functions/ecm.usb0
        echo "02${MAC}" > functions/ecm.usb0/dev_addr
        echo "12${MAC}" > functions/ecm.usb0/host_addr
    fi

    if [ -f /data/enable-usb-audio-gadget ]; then
        # Setup UAC2
        mkdir functions/uac2.usb0
        echo "3" > functions/uac2.usb0/c_chmask
        echo "2" > functions/uac2.usb0/c_ssize
        echo "48000" > functions/uac2.usb0/c_srate
        echo "3" > functions/uac2.usb0/p_chmask
        echo "2" > functions/uac2.usb0/p_ssize
        echo "48000" > functions/uac2.usb0/p_srate
        echo "8" > functions/uac2.usb0/req_number
        echo "${MODEL_NAME} Audio" > functions/uac2.usb0/function_name
    fi

    # Configuration
    mkdir configs/mod.1
    echo "0xc0" > configs/mod.1/bmAttributes # self-powerered
    echo "1" > configs/mod.1/MaxPower # 2mA

    mkdir configs/mod.1/strings/0x409
    echo "${MODEL_NAME}" > configs/mod.1/strings/0x409/configuration

    if [ ${WINDOWS_COMPAT_MODE} -eq 1 ]; then
        ln -s functions/rndis.usb0 configs/mod.1
        ln -s configs/mod.1 os_desc
    else
        ln -s functions/ecm.usb0 configs/mod.1
    fi
    if [ -f /data/enable-usb-audio-gadget ]; then
        ln -s functions/uac2.usb0 configs/mod.1
    else
        # FIXME on Duo with 5.17.12 usb gadget fails to activate if both audio and midi are enabled :/
        ln -s functions/midi.usb0 configs/mod.1
    fi

    # Ready to roll!
    /bin/find /sys/class/udc -type l -maxdepth 1 | /bin/tail -n 1 | /bin/awk 'sub("/sys/class/udc/","")' > UDC
else
    /sbin/modprobe g_ether
fi

# Bring up usb0 ethernet interface
/sbin/ifup usb0

# Restarts sshd service, needed after usb0 being active
/usr/bin/systemctl restart sshd

# On legacy units, g_ether waits for first USB to be plugged in
# As such, netmanager cannot start on boot, so do that now
if [ -f /data/enable-netmanager ] && [ "${USB_PLUGGED_IN_WAIT_REQUITED}" -eq 1 ]; then
    /usr/bin/systemctl start jack-netmanager
fi

exit 0
