#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

from subprocess import getoutput

# ------------------------------------------------------------------------------------------------------------
# Configuration

MOD_IP = "192.168.51.1"
MOD_USER = "root"
MOD_PASSWORD = "mod"

MOD_HOST_IP = "192.168.101.1"

# ------------------------------------------------------------------------------------------------------------
# Current device info

class DeviceInfo():
    __slots__ = [
        'model_name',              # "MOD Duo", "MOD Duo X", etc
        'model_type',              # full cpu model type, like "duox:aarch64-a53"
        'platform',                # "duo", "duox", etc
        'i2c_main_bus',            # number
        'cc_baud_rate',            # Control-Chain baud rate
        'cc_serial_port',          # Control-Chain serial port, including /dev prefix
        'hmi_baud_rate',           # HMI baud rate
        'hmi_serial_port',         # HMI serial port, including /dev prefix
        'midi_serial_port',        # MIDI serial port, including /dev prefix
        'pca9685_gpio_path',       # PCA9685 gpio path
        'spi2jack_path',           # "MODDUO", "DUOX", etc
        'jack2spi_path',           # "MODDUO", "DUOX", etc
        'soundcard',               # "MODDUO", "DUOX", etc
        'jackd_extra_args',        # empty by default, to be appended to jackd arguments
        'soundcard_loopback_name', # either "LOOPBACK" or "Loopback"
        'soundcard_has_hp_bypass', # boolean
        'soundcard_has_cv_stuff',  # boolean
        'soundcard_has_hw_bypass', # boolean
    ]

    def __str__(self):
        return "DeviceInfo({\n%s\n})" % (
            "\n".join(tuple("\t%s:\t%s" % (key, getattr(self, key)) for key in DeviceInfo.__slots__))
        )

    def __init__(self):

        (MODEL_NAME, MODEL_TYPE,
         PLATFORM, I2C_MAIN_BUS,
         CC_BAUD_RATE, CC_SERIAL_PORT,
         HMI_BAUD_RATE, HMI_SERIAL_PORT,
         MIDI_SERIAL_PORT, PCA9685_OE_GPIO_PATH,
         SPI2JACK_DEVICE, JACK2SPI_DEVICE,
         SOUNDCARD, JACK_DEVICE_EXTRA_ARGS) = getoutput("""
            source /etc/mod-hardware-descriptor.env
            echo ${MODEL_NAME}
            echo ${MODEL_TYPE}
            echo ${PLATFORM}
            echo ${I2C_MAIN_BUS}
            echo ${CC_BAUD_RATE}
            echo ${CC_SERIAL_PORT}
            echo ${HMI_BAUD_RATE}
            echo ${HMI_SERIAL_PORT}
            echo ${MIDI_SERIAL_PORT}
            echo ${PCA9685_OE_GPIO_PATH}
            echo ${SPI2JACK_DEVICE}
            echo ${JACK2SPI_DEVICE}
            echo ${SOUNDCARD}
            echo ${JACK_DEVICE_EXTRA_ARGS}
        """).split("\n")

        # pylint: disable=bad-whitespace
        self.model_name        = MODEL_NAME
        self.model_type        = MODEL_TYPE
        self.platform          = PLATFORM
        self.i2c_main_bus      = int(I2C_MAIN_BUS)
        self.cc_baud_rate      = int(CC_BAUD_RATE)
        self.cc_serial_port    = CC_SERIAL_PORT
        self.hmi_baud_rate     = int(HMI_BAUD_RATE)
        self.hmi_serial_port   = HMI_SERIAL_PORT
        self.midi_serial_port  = MIDI_SERIAL_PORT
        self.pca9685_gpio_path = PCA9685_OE_GPIO_PATH
        self.spi2jack_path     = SPI2JACK_DEVICE
        self.jack2spi_path     = JACK2SPI_DEVICE
        self.soundcard         = SOUNDCARD
        self.jackd_extra_args  = JACK_DEVICE_EXTRA_ARGS

        # special soundcard stuff
        if PLATFORM == "duo":
            self.soundcard_loopback_name = "LOOPBACK"
            self.soundcard_has_hp_bypass = True
            self.soundcard_has_cv_stuff  = False
            self.soundcard_has_hw_bypass = True
        elif PLATFORM == "duox":
            self.soundcard_loopback_name = "Loopback"
            self.soundcard_has_hp_bypass = False
            self.soundcard_has_cv_stuff  = True
            self.soundcard_has_hw_bypass = True
        else:
            self.soundcard_loopback_name = "LOOPBACK"
            self.soundcard_has_hp_bypass = False
            self.soundcard_has_cv_stuff  = False
            self.soundcard_has_hw_bypass = False
        # pylint: enable=bad-whitespace

        print(self)

devInfo = DeviceInfo()

# ------------------------------------------------------------------------------------------------------------
# Global stream access

class TextTestStream():
    def __init__(self):
        self.hmi_available = True
        self.from_pc = False
        self.stream = None

    def setStreamFromRunner(self, runner):
        self.stream = runner.stream

    def print(self, msg):
        if self.stream is None:
            return
        self.stream.writeln()
        self.stream.writeln("[SELF TEST ERROR] " + msg)
        self.stream.writeln()

testStream = TextTestStream()

# ------------------------------------------------------------------------------------------------------------
