#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

# usual stuff
import os

# additional stuff
from subprocess import Popen
from time import sleep

# local
from config import devInfo, testStream
from test_abstract import ControllerTestCase, ExtendedTestCase, kill_self, range_list, self_test_helper

# ------------------------------------------------------------------------------------------------------------

kAudioTestsRetryAttempts = 3

# ------------------------------------------------------------------------------------------------------------

class AutoTerminatingProcess():
    def __init__(self, proc):
        self.proc = proc
    def __del__(self):
        if self.proc is not None:
            self.proc.terminate()

# ------------------------------------------------------------------------------------------------------------

# Special test used if HMI is not available, so we need to show message to disconnect all cables here
class TestAudioShowDisconnectIfFirst(ControllerTestCase):
    def test_setup(self):
        if testStream.hmi_available:
            return

        # test if operator knows what to do (try 2 times)
        for i in range(2):
            try:
                msg = ("Self test mode.\n"
                       "Disconnect all cables\n"
                       "and select 'YES' to continue")
                if i != 0:
                    msg += "\nLast attempt."
                self.clearDisplay2()
                self.showYesNoDialog(msg)
                break
            # pylint: disable=bare-except
            except:
                if i == 0:
                    continue

            self.showText("ERROR", 0)
            self.showText("Must select 'YES'", 1)
            kill_self()

# ------------------------------------------------------------------------------------------------------------

class TestAudioCardsAvailable(ExtendedTestCase):
    def test_cards(self):
        try:
            self.assertTrue(os.path.exists("/proc/asound/cards"))
        except AssertionError as e:
            testStream.print("List of soundcards not available")
            raise e

        with open("/proc/asound/cards", 'r') as fh:
            content = fh.read().strip()

        try:
            self.assertTrue(bool(content))
        except AssertionError as e:
            testStream.print("No soundcards available")
            raise e

    def test_card_mod(self):
        device_ids = []

        for i in range(0, 10):
            path = "/proc/asound/card%i/id" % i
            if not os.path.exists(path):
                break
            with open(path, 'r') as fh:
                device_ids.append(fh.read().strip())

        try:
            self.assertIn(devInfo.soundcard, device_ids)
        except AssertionError as e:
            testStream.print("MOD codec soundcard not available")
            raise e

# ------------------------------------------------------------------------------------------------------------

class TestAudioCardMixerElements(ExtendedTestCase):
    kCaptureElements = ("PGA Gain", "Left Gain Stage", "Right Gain Stage")
    kPlaybackElements = ["DAC", devInfo.soundcard_loopback_name, "Headphone"]

    if devInfo.soundcard_has_hw_bypass:
        kPlaybackElements.insert(0, "Left True-Bypass")
        kPlaybackElements.insert(0, "Right True-Bypass")
    if devInfo.soundcard_has_hp_bypass:
        kPlaybackElements.insert(0, "AUX OUT MUX")
    if devInfo.soundcard_has_cv_stuff:
        kPlaybackElements.insert(0, "Headphone/CV Mode")
        kPlaybackElements.insert(0, "CV/Exp.Pedal Mode")
        kPlaybackElements.insert(0, "Exp.Pedal Mode")

    # Check if all capture elements exist
    def test_check_capture_elements(self):
        for element in self.kCaptureElements:
            try:
                self.assertCommand("amixer sget -D hw:%s '%s'" % (devInfo.soundcard, element))
            except AssertionError as e:
                testStream.print("Soundcard capture element '%s' missing" % (element,))
                raise e

    # Check if all playback elements exist
    def test_check_playback_elements(self):
        for element in self.kPlaybackElements:
            try:
                self.assertCommand("amixer sget -D hw:%s '%s'" % (devInfo.soundcard, element))
            except AssertionError as e:
                testStream.print("Soundcard playback element '%s' missing" % (element,))
                raise e

    # NOTE: these tests below are not enabled right now
    def _test_element_pga_gain(self):
        for i in range_list(48):
            try:
                self.assertCommand("amixer sset -D hw:%s 'PGA Gain' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard PGA Gain value '%i' failed" % (i,))
                raise e

    def _test_element_gain_stage(self):
        for i in range_list(3):
            try:
                self.assertCommand("amixer sset -D hw:%s 'Left Gain Stage' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard Left Gain Stage value '%i' failed" % (i,))
                raise e

            try:
                self.assertCommand("amixer sset -D hw:%s 'Right Gain Stage' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard Right Gain Stage value '%i' failed" % (i,))
                raise e

    def _test_element_aux_out_mux(self):
        if not devInfo.soundcard_has_hp_bypass:
            return
        for i in range_list(3):
            try:
                self.assertCommand("amixer sset -D hw:%s 'AUX OUT MUX' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard Headphone monitor value '%i' failed" % (i,))
                raise e

    def _test_element_dac(self):
        for i in range_list(255):
            try:
                self.assertCommand("amixer sset -D hw:%s 'DAC' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard DAC value '%i' failed" % (i,))
                raise e

    def _test_element_bypass(self):
        for s in ('on', 'off'):
            try:
                self.assertCommand("amixer sset -D hw:{} '{}' {}".format(devInfo.soundcard,
                                                                         devInfo.soundcard_loopback_name,
                                                                         s))
            except AssertionError as e:
                testStream.print("Soundcard Loopback value '%s' failed" % (s,))
                raise e

            if not devInfo.soundcard_has_hw_bypass:
                return
            try:
                self.assertCommand("amixer sset -D hw:%s 'Left True-Bypass' %s" % (devInfo.soundcard, s))
            except AssertionError as e:
                testStream.print("Soundcard Left True-Bypass value '%s' failed" % (s,))
                raise e

            try:
                self.assertCommand("amixer sset -D hw:%s 'Right True-Bypass' %s" % (devInfo.soundcard, s))
            except AssertionError as e:
                testStream.print("Soundcard Right True-Bypass value '%s' failed" % (s,))
                raise e

    def _test_element_headphone(self):
        for i in range_list(48):
            try:
                self.assertCommand("amixer sset -D hw:%s 'Headphone' %i" % (devInfo.soundcard, i))
            except AssertionError as e:
                testStream.print("Soundcard Headphone value '%i' failed" % (i,))
                raise e

# ------------------------------------------------------------------------------------------------------------

class JackdTestCase(ControllerTestCase):
    def setUp(self):
        ControllerTestCase.setUp(self)

        self._p = None
        self.proc = Popen(['jackd', '-R', '-P', '80',
                           '-d', 'alsa', '-d', 'hw:'+devInfo.soundcard,
                           '-r', '48000', '-p', '128', '-n', '2',
                           '-X', 'seq'] + devInfo.jackd_extra_args.split())

        try:
            self.assertIsNone(self.proc.poll())
        except AssertionError as e:
            testStream.print("JACK failed to start")
            raise e

        self._p = AutoTerminatingProcess(self.proc)
        try:
            self.assertCommand("jack_wait -w")
        except AssertionError as e:
            testStream.print("JACK failed to connect")
            raise e

        try:
            self.assertIsNone(self.proc.poll())
        except AssertionError as e:
            testStream.print("JACK process crashed")
            raise e

    def tearDown(self):
        self._p.proc = None
        self.proc.terminate()
        try:
            self.assertEqual(self.proc.wait(), 0)
        except AssertionError as e:
            testStream.print("JACK failed to stop")
            raise e

        ControllerTestCase.tearDown(self)

# ------------------------------------------------------------------------------------------------------------

class TestJackdServerUsage(JackdTestCase):
    def test_usage(self):
        try:
            self.assertCommandOutput("jack_bufsize", "128")
        except AssertionError as e:
            testStream.print("JACK buffer size request failed")
            raise e

        try:
            self.assertIsNone(self.proc.poll())
        except AssertionError as e:
            testStream.print("JACK process crashed")
            raise e

        try:
            self.assertCommand("jack_load mod-monitor")
        except AssertionError as e:
            testStream.print("Failed to load JACK mod-monitor client")
            raise e

        try:
            self.assertCommand("jack_load mod-midi-merger")
        except AssertionError as e:
            testStream.print("Failed to load JACK mod-midi-merger client")
            raise e

        try:
            self.assertCommand("jack_load mod-midi-broadcaster")
        except AssertionError as e:
            testStream.print("Failed to load JACK mod-midi-broadcaster client")
            raise e

        sleep(1)

        try:
            self.assertCommand("jack_unload mod-midi-broadcaster")
        except AssertionError as e:
            testStream.print("Failed to unload JACK mod-midi-broadcaster client")
            raise e

        try:
            self.assertCommand("jack_unload mod-midi-merger")
        except AssertionError as e:
            testStream.print("Failed to unload JACK mod-midi-merger client")
            raise e

        try:
            self.assertCommand("jack_unload mod-monitor")
        except AssertionError as e:
            testStream.print("Failed to unload JACK mod-monitor client")
            raise e

# ------------------------------------------------------------------------------------------------------------

class TestAudioFlow(JackdTestCase):
    def test_0_init(self):
        #make sure to test audio at full brightness, so we dont have interference.
        self.setMaxDisplayBrightness()

        # turn everything up!
        try:
            self.assertCommand("amixer sset -D hw:{} 'PGA Gain' 48".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set PGA Gain value")
            raise e

        # always sleep a small bit after changing pga gain
        sleep(0.1)

        try:
            self.assertCommand("amixer sset -D hw:{} 'Left Gain Stage' 0".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Left Gain Stage value")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Right Gain Stage' 0".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Right Gain Stage value")
            raise e

        if devInfo.soundcard_has_hp_bypass:
            try:
                self.assertCommand("amixer sset -D hw:{} 'AUX OUT MUX' 1".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Headphone monitor value")
                raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'DAC' 255".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set DAC value")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} '{}' off".format(devInfo.soundcard,
                                                                      devInfo.soundcard_loopback_name))
        except AssertionError as e:
            testStream.print("Failed to set Loopback value")
            raise e

        if devInfo.soundcard_has_hw_bypass:
            try:
                self.assertCommand("amixer sset -D hw:{} 'Left True-Bypass' off".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Left True-Bypass value")
                raise e
            try:
                self.assertCommand("amixer sset -D hw:{} 'Right True-Bypass' off".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Right True-Bypass value")
                raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone' 5".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone value")
            raise e

    # --------------------------------------------------------------------------------------------------------

    def test_1_grounded(self):
        try:
            self.assertCommand("amixer sset -D hw:{} 'PGA Gain' 24".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set PGA Gain to 50%")
            raise e

        # always sleep a small bit after changing pga gain
        sleep(0.1)

        self.showText("Testing device noise floor", 0)
        self.showText("Please wait...", )

        # set Gains stage
        try:
            self.assertCommand("amixer sset -D hw:%s 'Left Gain Stage' %i" % (devInfo.soundcard, 0))
            self.assertCommand("amixer sset -D hw:%s 'Right Gain Stage' %i" % (devInfo.soundcard, 0))
        except AssertionError as e:
            testStream.print("Failed to set GainStage value to Low")
            raise e

        for retry in range(kAudioTestsRetryAttempts):
            try:
                self.assertCommand(self_test_helper + " sine-wave zero_loopback gainStage_low 1.0 -999.0 -60.0 -60.0 50.0")
                break
            except AssertionError as e:
                if retry+1 == kAudioTestsRetryAttempts:
                    testStream.print("Audio Loopback Zero test failed after %i attempt(s)" % kAudioTestsRetryAttempts)
                    raise e
                print("Audio Loopback Zero test failed, retrying...")

        self.clearDisplay1()

    # --------------------------------------------------------------------------------------------------------

    def test_2_main_audio_loopback(self):
        while not self.showYesNoDialog("Connect audio cables.\n"
                                       "From 'out 1' to 'in 1' and\n"
                                       "From 'out 2' to 'in 2'\n\n"
                                       "Select 'YES' when ready.", False):
            pass
        self.clearDisplay1()
        self.clearDisplay2()
        self.showText("Testing audio loopback..", 1)

        gainStages = ["gainStage_low", "gainStage_mid", "gainStage_high"]

        if devInfo.platform == "duox":
            sineAmp = [0.8, 0.35, 0.1]
            maxNoiseFloor = -50.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 25.0
            maxSecondHarmonic = -20.0
        elif devInfo.platform == "duo":
            sineAmp = [0.5, 0.05, 0.01]
            maxNoiseFloor = -35.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 10.0
            maxSecondHarmonic = -35.0
        elif devInfo.platform == "dwarf":
            sineAmp = [0.4, 0.2, 0.07]
            maxNoiseFloor = -37.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 20.0
            maxSecondHarmonic = -35.0
        else:
            return

        try:
            self.assertCommand("amixer sset -D hw:{} 'PGA Gain' 24".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set PGA Gain to 50%")
            raise e

        # always sleep a small bit after changing pga gain
        sleep(0.1)

        #repeat test 3 times, each time with a different gain stage.
        for i in range(0, 3):
            # set PGA Gain
            try:
                self.assertCommand("amixer sset -D hw:%s 'Left Gain Stage' %i" % (devInfo.soundcard, i))
                self.assertCommand("amixer sset -D hw:%s 'Right Gain Stage' %i" % (devInfo.soundcard, i))
                self.clearDisplay1()
                if i == 0:
                    self.showText("Testing GainStage: Low", 0)
                    self.showText("Testing GainStage: Low", 0)
                elif i == 1:
                    self.showText("Testing GainStage: Mid", 0)
                    self.showText("Testing GainStage: Mid", 0)
                elif i == 2:
                    self.showText("Testing GainStage: High", 0)
                    self.showText("Testing GainStage: High", 0)
            except AssertionError as e:
                testStream.print("Failed to set GainStage value to " + str(i))
                raise e

            #for t in range(0, len(sineFreq)):
            if devInfo.platform != "dwarf":
                self.clearDisplay2()
            self.showText("Testing at freqs: " + str(int(sineFreq[0])) + "Hz, " + str(int(sineFreq[1]/1000)) + "KHz &" + str(int(sineFreq[2]/1000)) + "KHz\n\nPlease wait...", 1)
            self.showText("Testing at freqs: " + str(int(sineFreq[0])) + "Hz, " + str(int(sineFreq[1]/1000)) + "KHz &" + str(int(sineFreq[2]/1000)) + "KHz\n\nPlease wait...", 1)
            for retry in range(kAudioTestsRetryAttempts):
                try:
                    self.assertCommand(self_test_helper + " sine-freq-response audio_loopback " + str(gainStages[i]) + " " + str(sineAmp[i]) + " " + str(minFirstHarmonic) + " " + str(maxSecondHarmonic) + " " + str(maxNoiseFloor) + " " + str(sineFreq[0]) + " " + str(sineFreq[1]) + " " + str(sineFreq[2]))
                    break
                except AssertionError as e:
                    if retry+1 == kAudioTestsRetryAttempts:
                        testStream.print("Audio Loopback test failed after %i attempt(s)" % kAudioTestsRetryAttempts)
                        raise e
                    print("Audio Loopback test failed, retrying...")

        self.clearDisplay1()

    # --------------------------------------------------------------------------------------------------------

    def test_3_headphone_audio_loopback(self):
        # make sure we turn off CV mode first
        if devInfo.soundcard == "DUOX":
            try:
                self.assertCommand("amixer sset -D hw:{} 'Headphone/CV Mode' off".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Headphone/CV Mode to off")
                raise e

        while not self.showYesNoDialog("Connect headphone cable.\n\nSelect 'YES' when ready.", False):
            pass

        self.clearDisplay1()
        self.showText("Testing Headphone loopback..", 0)

        # Set 50% PGA Gain, true bypass is still enabled
        try:
            self.assertCommand("amixer sset -D hw:{} 'PGA Gain' 24".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set PGA Gain to 50%")
            raise e

        # always sleep a small bit after changing pga gain
        sleep(0.1)

        # Set gain stage to Low
        try:
            self.assertCommand("amixer sset -D hw:%s 'Left Gain Stage' %i" % (devInfo.soundcard, 1))
            self.assertCommand("amixer sset -D hw:%s 'Right Gain Stage' %i" % (devInfo.soundcard, 1))
        except AssertionError as e:
            testStream.print("Failed to set gain stage to Mid")
            raise e

        # Enable true bypass if applicable, make sure audio is outputting on headphones only
        if devInfo.soundcard_has_hw_bypass:
            try:
                self.assertCommand("amixer sset -D hw:{} 'Left True-Bypass' on".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Left True-Bypass to on")
                raise e
            try:
                self.assertCommand("amixer sset -D hw:{} 'Right True-Bypass' on".format(devInfo.soundcard))
            except AssertionError as e:
                testStream.print("Failed to set Right True-Bypass to on")
                raise e

        # Testing Headphone at 0%
        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone' 0".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone gain to 0%")
            raise e

        # Testing Headphone at 100%
        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone' 15".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone gain to 100%")
            raise e

        if devInfo.platform == "duox":
            sineAmp = 0.1
            maxNoiseFloor = -40.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 20.0
            maxSecondHarmonic = -17.0
        elif devInfo.platform == "duo":
            sineAmp = 0.01
            maxNoiseFloor = -50.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 10.0
            maxSecondHarmonic = -40.0
        elif devInfo.platform == "dwarf":
            sineAmp = 0.07
            maxNoiseFloor = -30.0
            sineFreq = [100.0, 1000.0, 10000.0]
            minFirstHarmonic = 20.0
            maxSecondHarmonic = -15.0
        else:
            return

        if devInfo.platform != "dwarf":
            self.clearDisplay2()

        self.showText("Testing at freqs: " + str(int(sineFreq[0])) + "Hz, " + str(int(sineFreq[1]/1000)) + "KHz &" + str(int(sineFreq[2]/1000)) + "KHz\n\nPlease wait...", 1)
        self.showText("Testing at freqs: " + str(int(sineFreq[0])) + "Hz, " + str(int(sineFreq[1]/1000)) + "KHz &" + str(int(sineFreq[2]/1000)) + "KHz\n\nPlease wait...", 1)
        for retry in range(kAudioTestsRetryAttempts):
            try:
                self.assertCommand(self_test_helper + " sine-freq-response headphones gainStage_mid " + str(sineAmp) + " " + str(minFirstHarmonic) + " " + str(maxSecondHarmonic) + " " + str(maxNoiseFloor) + " " + str(sineFreq[0]) + " " + str(sineFreq[1]) + " " + str(sineFreq[2]))
                break
            except AssertionError as e:
                if retry+1 == kAudioTestsRetryAttempts:
                    testStream.print("Headphone Loopback test failed after %i attempt(s)" % kAudioTestsRetryAttempts)
                    raise e
                print("Headphone Loopback test failed, retrying...")

        self.clearDisplay1()

    # --------------------------------------------------------------------------------------------------------

class TestAudioFlowSPDIF(JackdTestCase):
    def test_1_spdif_audio_loopback(self):
        while not self.showYesNoDialog("Connect spdif cable to the\n"
                                       "converter and power it with\n"
                                       "USB, then connect the converter\n"
                                       "outputs to the MDX inputs\n\n"
                                       "Select 'YES' when ready.", False):
            pass

        self.clearDisplay1()
        self.showText("Testing SPDIF loopback..", 0)

        # Set 50% PGA Gain, true bypass is still enabled
        try:
            self.assertCommand("amixer sset -D hw:{} 'PGA Gain' 24".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set PGA Gain to 50%")
            raise e

        # always sleep a small bit after changing pga gain
        sleep(0.1)

        # Set gain stage to Low
        try:
            self.assertCommand("amixer sset -D hw:%s 'Left Gain Stage' %i" % (devInfo.soundcard, 0))
            self.assertCommand("amixer sset -D hw:%s 'Right Gain Stage' %i" % (devInfo.soundcard, 0))
        except AssertionError as e:
            testStream.print("Failed to set gain stage to Low")
            raise e

        # Enable true bypass, make sure audio is outputting on headphones only
        try:
            self.assertCommand("amixer sset -D hw:{} 'Left True-Bypass' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Left True-Bypass to off")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Right True-Bypass' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Right True-Bypass to off")
            raise e

        # Testing spdif signal
        try:
            self.assertCommand("amixer sset -D hw:{} 'SPDIF Enable' on".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to enable SPDIF")
            raise e

        sineAmp = 0.6
        maxNoiseFloor = -50.0
        sineFreq = 100.0
        minFirstHarmonic = 20.0
        maxSecondHarmonic = -45.0

        self.clearDisplay2()

        for retry in range(kAudioTestsRetryAttempts):
            try:
                self.assertCommand(self_test_helper + " sine-wave-spdif spdif_loopback gainStage_low " + str(sineAmp) + " " + str(minFirstHarmonic) + " " + str(maxSecondHarmonic) + " " + str(maxNoiseFloor) + " " + str(sineFreq))
            except AssertionError as e:
                if retry+1 == kAudioTestsRetryAttempts:
                    testStream.print("SPDIF Loopback test failed after %i attempt(s)" % kAudioTestsRetryAttempts)
                    raise e
                print("SPDIF Loopback test failed, retrying...")

        self.clearDisplay1()
        self.clearDisplay2()

# ------------------------------------------------------------------------------------------------------------

tests = [
    TestAudioShowDisconnectIfFirst,
    TestAudioCardsAvailable,
    TestAudioCardMixerElements,
    TestJackdServerUsage,
    TestAudioFlow
]

if devInfo.soundcard == "DUOX":
    tests.append(TestAudioFlowSPDIF)

# ------------------------------------------------------------------------------------------------------------
