#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

# additional stuff
from time import sleep

# local
from config import devInfo, testStream
from test_abstract import ControllerTestCase

# ------------------------------------------------------------------------------------------------------------

class TestControlVoltageLoopback(ControllerTestCase):
    def setUp(self):
        ControllerTestCase.setUp(self)

        if devInfo.soundcard != "DUOX":
            return

        # Turn on CV ports
        try:
            self.assertCommand("amixer sset -D hw:{} 'CV/Exp.Pedal Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set CV/Exp.Pedal Mode to off")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone/CV Mode' on".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone/CV Mode to on")
            raise e

        #open CV I/O
        self.cv_in_1 = "{}/in_voltage0_raw".format(devInfo.spi2jack_path)
        self.cv_in_2 = "{}/in_voltage1_raw".format(devInfo.spi2jack_path)
        self.cv_out_1 = "{}/out_voltage0_raw".format(devInfo.jack2spi_path)
        self.cv_out_2 = "{}/out_voltage1_raw".format(devInfo.jack2spi_path)

    # Test connected
    def test_1_cv_loopback_on(self):
        if devInfo.soundcard != "DUOX":
            return

        while not self.showYesNoDialog("Connect a stereo jack from the CV input to the CV output.\n"
                                       "Select 'YES' when ready.", False):
            pass

        self.clearDisplay1()
        self.clearDisplay2()
        self.showText("Please wait...", 1)
        self.showText("Please wait...", 1)

        with open(self.cv_out_1, "w") as fh:
            fh.write("4095\n")
        with open(self.cv_out_2, "w") as fh:
            fh.write("4095\n")

        sleep(1)

        with open(self.cv_in_1, "r") as fh:
            inp1_val = fh.read()
        with open(self.cv_in_2, "r") as fh:
            inp2_val = fh.read()

        try:
            self.assertEqual((int(inp1_val) > 3000), True)
        except AssertionError as e:
            testStream.print("CV input 1 value too low during CV loopback")
            raise e

        try:
            self.assertEqual((int(inp2_val) > 3000), True)
        except AssertionError as e:
            testStream.print("CV input 2 value too low during CV loopback")
            raise e

    # Test disconnected
    def test_2_cv_loopback_off(self):
        if devInfo.soundcard != "DUOX":
            return

        # Testing no signal
        try:
            self.assertCommand("amixer sset -D hw:{} 'CV/Exp.Pedal Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set CV/Exp.Pedal Mode to off")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone/CV Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone/CV Mode to off")
            raise e

        with open(self.cv_out_1, "w") as fh:
            fh.write("0\n")
        with open(self.cv_out_2, "w") as fh:
            fh.write("0\n")

        sleep(1)

        with open(self.cv_in_1, "r") as fh:
            inp1_val = fh.read()
        with open(self.cv_in_2, "r") as fh:
            inp2_val = fh.read()

        try:
            self.assertEqual((int(inp1_val) < 50), True)
        except AssertionError as e:
            testStream.print("CV input 1 value too high (without CV loopback)")
            raise e

        try:
            self.assertEqual((int(inp2_val) < 50), True)
        except AssertionError as e:
            testStream.print("CV input 1 value too high (without CV loopback)")
            raise e

    # Test expression pedal
    def test_3_expression_pedal(self):
        if devInfo.soundcard != "DUOX":
            return

        while not self.showYesNoDialog("disconnect the stereo Jack\n"
                                       "Control voltage loopback cable\n"
                                       "Select 'YES' when ready.", False):
            pass

        self.clearDisplay2()
        self.showText("Please wait...", 1)
        self.showText("Please wait...", 1)

        try:
            self.assertCommand("amixer sset -D hw:{} 'CV/Exp.Pedal Mode' on".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set CV/Exp.Pedal Mode to on")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Exp.Pedal Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Exp.Pedal Mode to off")
            raise e

        sleep(1)

        with open(self.cv_in_1, "r") as fh:
            inp1_val = fh.read()
        with open(self.cv_in_2, "r") as fh:
            inp2_val = fh.read()

        try:
            self.assertEqual((int(inp2_val) > 3999), True)
        except AssertionError as e:
            testStream.print("CV input 2 value too low (during exp.pedal mode off)")
            raise e

        try:
            self.assertEqual((int(inp1_val) < 100), True)
        except AssertionError as e:
            testStream.print("CV input 1 value too high (during exp.pedal mode off)")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Exp.Pedal Mode' on".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Exp.Pedal Mode to on")
            raise e

        sleep(1)

        with open(self.cv_in_1, "r") as fh:
            inp1_val = fh.read()
        with open(self.cv_in_2, "r") as fh:
            inp2_val = fh.read()

        try:
            self.assertEqual((int(inp2_val) < 100), True)
        except AssertionError as e:
            testStream.print("CV input 2 value too high (during exp.pedal mode on)")
            raise e

        try:
            self.assertEqual((int(inp1_val) > 3999), True)
        except AssertionError as e:
            testStream.print("CV input 1 value too low (during exp.pedal mode on)")
            raise e

        # turn all off again
        try:
            self.assertCommand("amixer sset -D hw:{} 'Exp.Pedal Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Exp.Pedal Mode to off")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'Headphone/CV Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set Headphone/CV Mode to off")
            raise e

        try:
            self.assertCommand("amixer sset -D hw:{} 'CV/Exp.Pedal Mode' off".format(devInfo.soundcard))
        except AssertionError as e:
            testStream.print("Failed to set CV/Exp.Pedal Mode to off")
            raise e

# ------------------------------------------------------------------------------------------------------------

tests = [
    TestControlVoltageLoopback
]

# ------------------------------------------------------------------------------------------------------------
